/**
* Copyright 2007 - 2010 Skyway Software, Inc.
*/

package org.skyway.spring.util.dao.call.oracle;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

@SuppressWarnings("nls")
public abstract class OracleIntrospector {
	public abstract String getClassName();
	public abstract Object getInstance();
	
	protected Object findAndExecuteMethod(String methodName, Object... args){
		Class<?> classToExecute;
		Method method;
		
		try {
			classToExecute = findClassForName(getClassName());
		
			method = findMethod(classToExecute, methodName, args);

			return method.invoke(getInstance(), args);
		} catch (Exception e) {
			throw new RuntimeException("Unable to execute method [" + methodName + "] on class [" + getClassName() + "].  Make sure the oracle jar is on your classpath.", e);
		}
	}

	protected Object findAndExecuteConstructor(Object... args){
		Constructor<?> constructor;
		Class<?> classToExecute;
		
		try {
			classToExecute = findClassForName(getClassName());
		
			constructor = findConstructor(classToExecute, args);

			return constructor.newInstance(args);
		} catch (Exception e) {
			throw new RuntimeException("Unable to instantiate instance of class [" + getClassName() + "].  Make sure the oracle jar is on your classpath.", e);
		}
	}
	
	public static Class<?> findClassForName(String className){
		try {
			return Class.forName(className);
		} catch (ClassNotFoundException e) {
			throw new RuntimeException("Unable to find class [" + className + "].  Make sure the oracle jar is on your classpath.", e);
		}
	}

	public static Method findMethod(Class<?> classDefinition, String methodName, Object... args){
		Class<?>[] argumentTypes = new Class<?>[args.length];
		int index = 0;
		
		try {
			for (Object argument : args){
				argumentTypes[index++] = argument.getClass();
			}
			
			return classDefinition.getMethod(methodName, argumentTypes);
		} catch (NoSuchMethodException e) {
			throw new RuntimeException("Unable to find method [" + methodName + "] on class [" + classDefinition.getName() + "].  Make sure the oracle jar is on your classpath.", e);
		}
	}

	public static Constructor<?> findConstructor(Class<?> classDefinition, Object... args){
		Class<?>[] argumentTypes = new Class<?>[args.length];
		int index = 0;
		
		try {
			for (Object argument : args){
				argumentTypes[index++] = argument.getClass();
			}
			
			return classDefinition.getConstructor(argumentTypes);
		} catch (NoSuchMethodException e) {
			throw new RuntimeException("Unable to find constructor on class [" + classDefinition.getName() + "].  Make sure the oracle jar is on your classpath.", e);
		}
	}
}
